package envoy.server.data;

import java.util.Date;
import java.util.Set;
import java.util.stream.Collectors;

import javax.persistence.*;

import envoy.data.User.UserStatus;

/**
 * This class enables the storage of user specific data inside a database using
 * Hibernate. Its objects will be referred to as database users as opposed to
 * the common user objects present on both the client and the server.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>User.java</strong><br>
 * Created: <strong>02.01.2020</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-alpha
 */
@Entity
@Table(name = "users")
@NamedQueries(
	{ @NamedQuery(query = "SELECT u FROM User u WHERE u.name = :name", name = "getUserByName"), @NamedQuery(
		query = "SELECT u.contacts FROM User u WHERE u = :user",
		name = "getContactsOfUser"
	), @NamedQuery(
		query = "SELECT u FROM User u WHERE (lower(u.name) LIKE lower(:searchPhrase) AND u <> :context AND NOT :context in elements(u.contacts))",
		name = "searchUsers"
	) }
)
public class User extends Contact {

	private byte[] passwordHash;

	@Temporal(TemporalType.TIMESTAMP)
	private Date lastSeen;

	private UserStatus status;

	/**
	 * {@inheritDoc}
	 */
	@Override
	public envoy.data.User toCommon() {
		return new envoy.data.User(id, name, status, contacts.parallelStream().map(Contact::toFlatCommon).collect(Collectors.toSet()));
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected envoy.data.User toFlatCommon() { return new envoy.data.User(id, name, status, Set.of()); }

	/**
	 * @return the password hash
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public byte[] getPasswordHash() { return passwordHash; }

	/**
	 * @param passwordHash the password hash to set
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public void setPasswordHash(byte[] passwordHash) { this.passwordHash = passwordHash; }

	/**
	 * @return the last date the user has been online
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public Date getLastSeen() { return lastSeen; }

	/**
	 * @param lastSeen the latest date at which the user has been online to set
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public void setLastSeen(Date lastSeen) { this.lastSeen = lastSeen; }

	/**
	 * @return the status
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public UserStatus getStatus() { return status; }

	/**
	 * @param status the status to set
	 * @since Envoy Server Standalone v0.1-alpha
	 */
	public void setStatus(UserStatus status) { this.status = status; }
}
